#!/bin/sh

if test -d ~/Qt/5.3/clang_64/bin; then
    export PATH=$PATH:~/Qt/5.3/clang_64/bin
fi

if [ $# != 3 ]; then
    echo
    echo "Usage: mkdmg appname kind version"
    echo
    echo "appname: [MediaInfo | mi] or [MediaConch | mc]"
    echo "kind: [CLI | cli] or [GUI | gui]"
    echo
    exit 1
fi

APPNAME="$1"
KIND="$2"
VERSION="$3"

if [ "$KIND" = "CLI" ] || [ "$KIND" = "cli" ]; then
    KIND="CLI"
elif [ "$KIND" = "GUI" ] || [ "$KIND" = "gui" ]; then
    KIND="GUI"
else
    echo
    echo "KIND must be either [MediaInfo | mi] or [MediaConch | mc]"
    echo
    exit 1
fi
if [ "$APPNAME" = "MediaInfo" ] || [ "$APPNAME" = "mediainfo" ] || [ "$APPNAME" = "mi" ]; then
    APPNAME="MediaInfo"
elif [ "$APPNAME" = "MediaConch" ] || [ "$APPNAME" = "mediaconch" ] || [ "$APPNAME" = "mc" ]; then
    APPNAME="MediaConch"
else
    echo
    echo "APPNAME must be either [CLI | cli] or [GUI | gui]"
    echo
    exit 1
fi

APPNAME_lower=`echo ${APPNAME} |awk '{print tolower($0)}'`
KIND_lower=`echo ${KIND} |awk '{print tolower($0)}'`
SIGNATURE="MediaArea.net"
FILES="tmp-${APPNAME}_${KIND}"
TEMPDMG="tmp-${APPNAME}_${KIND}.dmg"
FINALDMG="${APPNAME}_${KIND}_${VERSION}_Mac.dmg"
WD=`pwd`

# Clean up
rm -fr "${FILES}-Root"
rm -fr "${FILES}"
rm -f "${APPNAME}.pkg"
rm -f "${TEMPDMG}"
rm -f "${FINALDMG}"

echo
echo ========== Create the package ==========
echo

mkdir -p "${FILES}/.background"
cp ../../License.html "${FILES}"
cp "../../Release/ReadMe_${KIND}_Mac.txt" "${FILES}/ReadMe.txt"
cp "../../History_${KIND}.txt" "${FILES}/History.txt"
cp Logo_White.icns "${FILES}/.background"

if [ "$KIND" = "CLI" ]; then

    mkdir -p "${FILES}-Root/usr/local/bin"
    cp "../GNU/CLI/${APPNAME_lower}" "${FILES}-Root/usr/local/bin"
    codesign --force --verbose --sign "Developer ID Application: ${SIGNATURE}" "${FILES}-Root/usr/local/bin/${APPNAME_lower}"

    pkgbuild --root "${FILES}-Root" --identifier "net.mediaarea.${APPNAME_lower}.mac-${KIND_lower}" --sign "Developer ID Installer: ${SIGNATURE}" --version "${VERSION}" "${FILES}/${APPNAME_lower}.pkg"
    codesign --force --verbose --sign "Developer ID Application: ${SIGNATURE}" "${FILES}/${APPNAME_lower}.pkg"

fi

if [ "$KIND" = "GUI" ]; then

    if [ "$APPNAME" = "MediaInfo" ]; then

        mkdir -p "${FILES}/${APPNAME}.app/Contents/MacOS"
        mkdir "${FILES}/${APPNAME}.app/Contents/Resources"
        cp "../GNU/GUI/${APPNAME_lower}-${KIND_lower}" "${FILES}/${APPNAME}.app/Contents/MacOS/${APPNAME}"
        cp Info.plist "${FILES}/${APPNAME}.app/Contents"
        sed -i '' -e "s/VERSION/${VERSION}/g" "${FILES}/${APPNAME}.app/Contents/Info.plist"
        echo -n 'APPL????' > "${FILES}/${APPNAME}.app/Contents/PkgInfo"
        cp ${APPNAME}.icns "${FILES}/${APPNAME}.app/Contents/Resources"
        
        codesign --force --verbose --sign "Developer ID Application: ${SIGNATURE}" "${FILES}/${APPNAME}.app/Contents/MacOS/${APPNAME}"
        codesign --force --verbose --sign "Developer ID Application: ${SIGNATURE}" "${FILES}/${APPNAME}.app"

    fi
    
    if [ "$APPNAME" = "MediaConch" ]; then

        cp -r "../Qt/${APPNAME}.app" "${FILES}/"
        sed -i '' -e "s/VERSION/${VERSION}/g" "${FILES}/${APPNAME}.app/Contents/Info.plist"
        macdeployqt "${FILES}/${APPNAME}.app"

        # Qt 5.3 doesn’t handle the new version of Apple
		# signatures (Mac 10.9+), so we must determin which Qt
		# frameworks is used by MediaConch and sign them manually.
        for FRAMEWORK in `ls "${FILES}/${APPNAME}.app"/Contents/Frameworks |grep framework | sed "s/\.framework//"`
        do

            echo
            echo
            echo Signing ${FRAMEWORK}...
            echo
            cd "${FILES}/${APPNAME}.app/Contents/Frameworks/${FRAMEWORK}.framework"
            # Despite their misleading names, these directories
			# generated by macdeployqt must be deleted, or codesign
			# will fail.
            rm -fr _CodeSignature
            rm -fr Versions/Current/_CodeSignature
            cd Versions
                # The trailing slash saga continues… codesign will
                # fail with "ln -s 5/ Current".
                ln -s 5 Current
            cd ..
            cp "${HOME}/Qt/5.3/clang_64/lib/${FRAMEWORK}.framework/Contents/Info.plist" Resources
            mv Resources Versions/Current
            ln -s Versions/Current/${FRAMEWORK}
            ln -s Versions/Current/Resources Resources
            if [ "$FRAMEWORK" = "QtPrintSupport" ] || [ "$FRAMEWORK" = "QtQuick" ]; then
                TMP=$(sed 's/_debug//g' Resources/Info.plist)
                echo "$TMP" > Resources/Info.plist
            fi
            cd $WD
            codesign --no-strict --force --verbose --sign "Developer ID Application: ${SIGNATURE}" "${FILES}/${APPNAME}.app/Contents/Frameworks/${FRAMEWORK}.framework"

        done
    
        echo
        echo
        echo Signing the application...
        echo
        codesign --deep --no-strict --force --verbose --sign "Developer ID Application: ${SIGNATURE}" "${FILES}/${APPNAME}.app/Contents/MacOS/${APPNAME}"
        codesign --deep --no-strict --force --verbose --sign "Developer ID Application: ${SIGNATURE}" "${FILES}/${APPNAME}.app"
            
        echo
        echo
        echo Verifying the signature...
        echo
        codesign --verify --deep --verbose=1 "${FILES}/${APPNAME}.app/Contents/MacOS/${APPNAME}"
    
        echo
        echo

        # Apparently not needed with Qt
        #productbuild --component "${FILES}/${APPNAME}.app" /Applications --sign "3rd Party Mac Developer Installer: ${SIGNATURE}" --product "${FILES}/${APPNAME}.app/Contents/Info.plist" "${APPNAME}.pkg"
    
    fi
fi

echo
echo ========== Create the disk image ==========
echo

# Check if an old image isn't already attached
DEVICE=$(hdiutil info |grep -B 1 "/Volumes/${APPNAME}" |egrep '^/dev/' | sed 1q | awk '{print $1}')
test -e "$DEVICE" && hdiutil detach -force "${DEVICE}"

hdiutil create "${TEMPDMG}" -ov -format UDRW -volname "${APPNAME}" -srcfolder "${FILES}"
DEVICE=$(hdiutil attach -readwrite -noverify "${TEMPDMG}" | egrep '^/dev/' | sed 1q | awk '{print $1}')
sleep 2

cd "/Volumes/${APPNAME}"
if [ "$KIND" = "GUI" ]; then
    ln -s /Applications
fi
test -e .DS_Store && rm -fr .DS_Store
cd - >/dev/null

. osascript_${APPNAME}_${KIND}.sh
osascript_Function

hdiutil detach "${DEVICE}"
sleep 2

echo
echo ========== Convert to compressed image ==========
echo
hdiutil convert "${TEMPDMG}" -format UDBZ -o "${FINALDMG}"

# Useless since the dmg will transit on no HFS+ partition (at least
# on the linux server)
#codesign --force --verbose --sign "Developer ID Application: ${SIGNATURE}" "${FINALDMG}"

unset -v APPNAME APPNAME_lower KIND KIND_lower VERSION SIGNATURE
unset -v FILES TEMPDMG FINALDMG WD FRAMEWORK DEVICE TMP
